package zero1hd.rhythmbullet.audio;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.utils.FloatArray;

import edu.emory.mathcs.jtransforms.fft.FloatFFT_1D;
import zero1hd.rhythmbullet.util.MiniEvents;
import zero1hd.rhythmbullet.util.MiniSender;

public class AudioAnalyzer {
	private boolean containsData;
	private boolean finalized;
	FloatFFT_1D fft;
	public AudioData audioData;

	float[] audioPCM;
	float[] spectrum;
	float[] lastSpectrum;
	
	Runnable analysisAlgorithm;
	Runnable thresholdCalculator;
	
	int bassBinBegin;
	int bassBinEnd;
	private FloatArray bassSpectralFlux = new FloatArray();
	private FloatArray bassThreshold = new FloatArray();
	private FloatArray bassPrunned = new FloatArray();
	private FloatArray bassPeaks = new FloatArray();
	private float bassMaxValue;
	private float bassAvg;
	float bassThresholdMultiplier;
	int bassThresholdCalcRange;
	
	int mBinBegin;
	int mBinEnd;
	private FloatArray mSpectralFlux = new FloatArray();
	private FloatArray mThreshold = new FloatArray();
	private FloatArray mPrunned = new FloatArray();
	private FloatArray mPeaks = new FloatArray();
	private float mMaxValue;
	private float mAvg; 
	float mThresholdMultiplier;
	int mThresholdCalcRange;
	
	int umBinBegin;
	int umBinEnd;
	private FloatArray umSpectralFlux = new FloatArray();
	private FloatArray umThreshold = new FloatArray();
	private FloatArray umPrunned = new FloatArray();
	private FloatArray umPeaks = new FloatArray();
	private float umMaxValue;
	private float umAvg; 
	float umThresholdMultiplier;
	int umThresholdCalcRange;


	public volatile MiniSender sender;
	
	private float avgSPB;
	
	int PUID;
	boolean work;
	private volatile int progress;
	private float secondsPerWindow;
	public AudioAnalyzer() {
		sender = new MiniSender();
		
		analysisAlgorithm = new Runnable() {

			@Override
			public void run() {
				progress = 0;
				int tasksDone = 0;
				int totalTasks = audioData.getSampleCount()/audioData.getReadWindowSize();
				
				bassThresholdMultiplier = 1.5f;
				mThresholdMultiplier = 1.4f;
				umThresholdMultiplier = 1.4f;
				
				bassBinBegin = 0;
				bassBinEnd = 12;
				
				mBinBegin = 50;
				mBinEnd = 250;
				
				umBinBegin = 350;
				umBinEnd = 513;
				
				bassThresholdCalcRange = thresholdRangeCalc(0.27f);
				mThresholdCalcRange = thresholdRangeCalc(0.4f);
				umThresholdCalcRange = thresholdRangeCalc(0.4f);
				
				Gdx.app.debug("Read freq", String.valueOf(audioData.getFormat().getSampleRate()));
				Gdx.app.debug("Using following bin ranges", "\nBass freq begin: " + bassBinBegin + "\nBass freq end: " + bassBinEnd + "\nMain freq begin: " + umBinBegin + "\nMain freq end: " + umBinEnd);

				
				Gdx.app.debug("Threshold Calc Range UM", String.valueOf(umThresholdCalcRange));
				Gdx.app.debug("Threshold Calc Range M", String.valueOf(umThresholdCalcRange));
				Gdx.app.debug("Threshold Calc Range Bass", String.valueOf(bassThresholdCalcRange));
				
				fft = new FloatFFT_1D(audioData.getReadWindowSize());
				int seedDigit = 0;
				while (audioData.readSamples(audioPCM) > 0 && work) {
					
					fft.realForward(audioPCM);
					
					//Building a PUID (Pseudo unique ID)
					if (tasksDone == (seedDigit*totalTasks/9)) {
						float avg = 0;
						for (int frame = 0; frame < spectrum.length; frame++) {
							avg += spectrum[frame];
						}
						avg /= spectrum.length;
						if (avg < 0) {
							avg *= -1f;
						}
						PUID +=(int) Math.pow(10, 9-seedDigit) * ((int)(avg*1000f)-(int)(avg*100f)*10);
						seedDigit ++;
					}
					
					System.arraycopy(spectrum, 0, lastSpectrum, 0, spectrum.length);
					System.arraycopy(audioPCM, 0, spectrum, 0, spectrum.length);
					
					float fluxVal;
					//bass detection
					fluxVal = 0;
					for (int i = bassBinBegin; i < bassBinEnd && work; i++) {
						fluxVal += ((spectrum[i] - lastSpectrum[i])) < 0
								? 0 : (spectrum[i] - lastSpectrum[i]);
					}
					bassSpectralFlux.add(fluxVal);
					
					//m detection
					fluxVal = 0;
					for (int i = mBinBegin; i < mBinEnd && work; i++) {
						fluxVal += ((spectrum[i] - lastSpectrum[i])) < 0
								? 0 : (spectrum[i] - lastSpectrum[i]);
					}
					mSpectralFlux.add(fluxVal);
					
					//um detection
					fluxVal = 0;
					for (int i = umBinBegin; i < umBinEnd && work; i++) {
						fluxVal += ((spectrum[i] - lastSpectrum[i])) < 0
								? 0 : (spectrum[i] - lastSpectrum[i]);
					}
					umSpectralFlux.add(fluxVal);
					tasksDone++;
					progress = (int) (100f*tasksDone/totalTasks);
					sender.send(MiniEvents.ANALYZER_ITERATED);
				}
				
				if (work) {
					Gdx.app.debug("Audio Analyzer", "Done getting spectral flux.");
					Gdx.app.debug("Audio Analyzer", "window count: " + bassSpectralFlux.size);
					shrinkData();
					containsData = true;
					Gdx.app.debug("Audio Analyzer", "USING SEED: " + PUID);
					sender.send(MiniEvents.SPECTRAL_FLUX_DONE);
				}
			}
		};
		
		thresholdCalculator = new Runnable() {
			@Override
			public void run() {
				Gdx.app.debug("Audio Analyzer", "beginning thrshold calc.");
				//threshold calculation
				for (int i = 0; i < umSpectralFlux.size && work; i++) {
					int start = Math.max(0, i - umThresholdCalcRange/2);
					int end = Math.min(umSpectralFlux.size - 1, i + umThresholdCalcRange/2);
					
					float average = 0;
					for (int j = start; j <= end; j++) {
						average += bassSpectralFlux.get(j);
					}
					average /= (end - start);
					bassThreshold.add(average * bassThresholdMultiplier);
					
					average = 0;
					for (int j = start; j <= end; j++) {
						average+= mSpectralFlux.get(j);
					}
					average /= (end - start);
					mThreshold.add(average*mThresholdMultiplier);
					
					average = 0;
					for (int j = start; j <= end; j++) {
						average+= umSpectralFlux.get(j);
					}
					average /= (end - start);
					umThreshold.add(average*umThresholdMultiplier);
				}
				
				Gdx.app.debug("Audio Analyzer", "Threshold calculated.");

				
				//pruning data
				float prunnedCurrentVal;
				for (int i = 0; i < umSpectralFlux.size && work; i++) {
					prunnedCurrentVal = bassSpectralFlux.get(i) - bassThreshold.get(i);
					if (prunnedCurrentVal >= 0) {
						bassPrunned.add(prunnedCurrentVal);
					} else {
						bassPrunned.add(0);
					}
					
					prunnedCurrentVal = mSpectralFlux.get(i) - mThreshold.get(i);
					if (prunnedCurrentVal >= 0 ) {
						mPrunned.add(prunnedCurrentVal);
					} else {
						mPrunned.add(0);
					}
					
					prunnedCurrentVal = umSpectralFlux.get(i) - umThreshold.get(i);
					if (prunnedCurrentVal >= 0 ) {
						umPrunned.add(prunnedCurrentVal);
					} else {
						umPrunned.add(0);
					}
				}
				Gdx.app.debug("Audio Analyzer", "Data prunned.");

				secondsPerWindow = audioData.getReadWindowSize()/audioData.getFormat().getSampleRate();
				//peak detection
				
				int lastID = 0;
				float bassBeats = 0;
				float mBeats = 0;
				float umBeats = 0;
				avgSPB = -1f;
				for (int i = 0; i < umPrunned.size-1 && work; i++) {
					bassPeaks.add((bassPrunned.get(i) > bassPrunned.get(i+1) ? bassPrunned.get(i) : 0f));
					if (bassPeaks.get(i) > bassMaxValue) {
						bassMaxValue = bassPeaks.get(i);
					}
					
					mPeaks.add((mPrunned.get(i) > mPrunned.get(i+1) ? mPrunned.get(i) : 0f));
					if (mPeaks.get(i) > mMaxValue) {
						mMaxValue = mPeaks.get(i);
					}
					
					umPeaks.add((umPrunned.get(i) > umPrunned.get(i+1) ? umPrunned.get(i) : 0f));
					if (umPeaks.get(i) > umMaxValue) {
						umMaxValue = umPeaks.get(i);
					}
					
					

					if (avgSPB != -1) {
						if (bassPeaks.get(i) == 0) {
							avgSPB ++;
						} else {
							lastID = i;
						}
					} else if (bassPeaks.get(i) != 0) {
						avgSPB = 0;
					}

					if (bassPeaks.get(i) != 0) {
						bassAvg += bassPeaks.get(i);
						bassBeats++;
						
					}
					
					if (mPeaks.get(i) != 0) {
						mAvg += mPeaks.get(i);
						mBeats++;
					}
					
					if (umPeaks.get(i) != 0) {
						umAvg += umPeaks.get(i);
						umBeats++;
					}
					
				}
				
				//then we minus one from the beats so it actually works out
				avgSPB -= bassPrunned.size-lastID;
				avgSPB *= secondsPerWindow;
				avgSPB /= bassBeats;
				Gdx.app.debug("Audio Analyzer", "Avg SPB: " + avgSPB);

				bassAvg /= bassBeats;
				mAvg /= mBeats;
				umAvg /= umBeats;
				Gdx.app.debug("Audio Analyzer", "Avg bass: " + bassAvg);
				Gdx.app.debug("Audio Analyzer", "Avg M: " + mAvg);
				Gdx.app.debug("Audio Analyzer", "Avg UM: " + umAvg);
				
				if (work) {
					finalized = true;
					
					sender.send(MiniEvents.MUSIC_DATA_CLEANED);
				}
			}
		};
		
	}
	
	public void shrinkData() {
		bassSpectralFlux.shrink();
		bassThreshold.shrink();
		bassPrunned.shrink();
		bassPeaks.shrink();
		
		mSpectralFlux.shrink();
		mThreshold.shrink();
		mPrunned.shrink();
		mPeaks.shrink();
		
		umSpectralFlux.shrink();
		umThreshold.shrink();
		umPrunned.shrink();
		umPeaks.shrink();
	}
	
	public void startAnalyticalThread(AudioData audiofile) {
		audioPCM = new float[audiofile.getReadWindowSize()];
		spectrum = new float[(audiofile.getReadWindowSize()/2)+1];
		lastSpectrum = new float[(audiofile.getReadWindowSize()/2)+1];
		this.audioData = audiofile;
		work = true;
		Thread analyticalThread = new Thread(analysisAlgorithm);
		analyticalThread.start();
	}
	
	public void runThresholdCleaning(float rangeModifier) {
		this.bassThresholdMultiplier -= rangeModifier;
		this.umThresholdMultiplier -= rangeModifier;
		work = true;
		Thread thresholdClean = new Thread(thresholdCalculator);
		thresholdClean.start();
	}
	
	public void runThresholdCleaning() {
		Thread thresholdClean = new Thread(thresholdCalculator);
		thresholdClean.start();
	}
	
	public FloatArray getBassPeaks() {
		return bassPeaks;
	}
	public FloatArray getmPeaks() {
		return mPeaks;
	}
	public FloatArray getUMPeaks() {
		return umPeaks;
	}
	
	private int thresholdRangeCalc(float durationOfRange) {
		return (int) (durationOfRange/(audioData.getReadWindowSize()/audioData.getFormat().getSampleRate()));
	}
	
	public float getBassMaxValue() {
		return bassMaxValue;
	}
	
	public float getmMaxValue() {
		return mMaxValue;
	}
	
	public float getUMMaxValue() {
		return umMaxValue;
	}
	
	public int getReadIndex() {
		if (audioData.getReadIndex() < umPeaks.size) {
			return audioData.getReadIndex();
		} else {
			return 0;
		}
	}
	
	public boolean containsData() {
		return containsData;
	}
	
	public synchronized int getProgress() {
		return progress;
	}
	
	public boolean isFinalized() {
		return finalized;
	}
	
	public void stop() {
		work = false;
	}
	
	public int getPUID() {
		return PUID;
	}
	
	public AudioData getAudioData() {
		return audioData;
	}
	
	public float getAvgSPB() {
		return avgSPB;
	}
	
	public float getsecondsPerWindow() {
		return secondsPerWindow;
	}
	
	public float getBassAvg() {
		return bassAvg;
	}
	
	public float getUmAvg() {
		return umAvg;
	}
	
	public float getmAvg() {
		return mAvg;
	}
}
