package zero1hd.rhythmbullet.audio.map;

import org.apache.commons.math3.random.MersenneTwister;

import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.utils.FloatArray;

import zero1hd.rhythmbullet.RhythmBullet;
import zero1hd.rhythmbullet.audio.AudioAnalyzer;
import zero1hd.rhythmbullet.entity.EntityManager;
import zero1hd.rhythmbullet.entity.coordinator.CoordinatorManager;
import zero1hd.rhythmbullet.util.MiniEvents;
import zero1hd.rhythmbullet.util.MiniSender;

public class RhythmMapAlgorithm implements Runnable {
	private MiniSender sender;
	
	private EntityManager em;
	private CoordinatorManager cm;
	
	private FloatArray bassPeaks;
	private FloatArray mPeaks;
	private FloatArray umPeaks;
	private MersenneTwister rand;
	private GamePlayMap map;
	
	private float avgBass;
	private float avgUM;
	private float avgSPB;
	private float umMax, bassMax;
	
	private float speedMod, healthMod, difficultyMod;
	
	private float windowPerSecond;
	
	private volatile int progress;
	public RhythmMapAlgorithm(EntityManager em, CoordinatorManager cm, AudioAnalyzer analyzer, float speedMod, float healthMod, float difficultyMod) {
		this.cm = cm;
		this.em = em;
		
		sender = new MiniSender();
		
		bassPeaks = analyzer.getBassPeaks();
		mPeaks = analyzer.getmPeaks();
		umPeaks = analyzer.getUMPeaks();
		map = new GamePlayMap(analyzer.getAudioData(), umPeaks.size);
		rand = new MersenneTwister(analyzer.getPUID());
		avgSPB = analyzer.getAvgSPB();
		windowPerSecond = 1/analyzer.getsecondsPerWindow();
		
		this.speedMod = speedMod;
		this.healthMod = healthMod;
		this.difficultyMod = difficultyMod;
		
		umMax = analyzer.getUMMaxValue();
		avgUM = analyzer.getUmAvg();

		bassMax = analyzer.getBassMaxValue();
		avgBass = analyzer.getBassAvg();
	}
	
	
	EntitySpawnInfo esi;
	@Override
	public void run() {
		map.beginBuild();
		for (int index = 0; index < bassPeaks.size; index++) {
			if (bassPeaks.get(index) != 0 || umPeaks.get(index) != 0) {
				if (bassPeaks.get(index) != 0) {
					//If there is a value of some sorts that is not zero, we generate some beat for the map
					if (index > 1.5*windowPerSecond && bassPeaks.get(index) > avgBass) {
						//If bass peak is greater than the bass peak average, then:
						int normalIndexPos = map.setIndex((int) (map.getIndex() - windowPerSecond*1.5f));
						float waitTime = 1.5f;
						float endRadius = (bassPeaks.get(index)/bassMax)*(RhythmBullet.GAME_AREA_HEIGHT/4f);
						
						esi = map.addEntity(em.voidCircle, null);
						esi.parameters.put("warningTime", waitTime);
						esi.parameters.put("endRadius", endRadius);
						esi.parameters.put("growthRate", endRadius/(avgSPB*0.8f));
						esi.parameters.put("x", rand.nextFloat()*RhythmBullet.GAME_AREA_WIDTH);
						esi.parameters.put("y", rand.nextFloat()*RhythmBullet.GAME_AREA_HEIGHT);
						
						map.setIndex(normalIndexPos);
					} else {
						
					}
				}
				
				if (umPeaks.get(index) != 0) {
					if (umPeaks.get(index) >= avgUM) {
						//If upper midrange peaks are greater than average, the:
						int spawnLocations = (RhythmBullet.GAME_AREA_WIDTH-8)/8;
						
						esi = map.addEntity(em.bar, null);
						esi.parameters.put("x", (float) (MathUtils.round(rand.nextFloat()*spawnLocations)*8));
						esi.parameters.put("rate", (8f/avgSPB)*speedMod);
						
					} else {
						float xSpawnLocation = (rand.nextFloat()*(RhythmBullet.GAME_AREA_WIDTH-2))+1;
						
						esi = map.addEntity(em.pellet, null);
						esi.parameters.put("x", xSpawnLocation);
						esi.parameters.put("y", RhythmBullet.GAME_AREA_HEIGHT-0.25f);
						esi.parameters.put("angle", 140*rand.nextFloat()+200f);
						esi.parameters.put("speed", (RhythmBullet.GAME_AREA_HEIGHT/4f)/avgSPB);
						
					}
				}
				
				
				if (rand.nextFloat() < 0.15f) {
					switch(rand.nextInt(10)) {
					case 0:
						break;
					}
				}
			}
			map.nextWindowData();

			progress = MathUtils.round(100f*index/bassPeaks.size);
			
			sender.send(MiniEvents.MAPGEN_ITERATED);
		}
		map.endBuild();
		sender.send(MiniEvents.MAP_GENERATED);
	}
	
	public synchronized GamePlayMap getMap() {
		return map;
	}
	
	public synchronized int getProgress() {
		return progress;
	}
	
	public MiniSender getSender() {
		return sender;
	}
	
	public EntityManager getEm() {
		return em;
	}
}
