package zero1hd.rhythmbullet.ui.stages;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.ImageButton;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;

import zero1hd.rhythmbullet.ui.builders.HealthBar;
import zero1hd.rhythmbullet.ui.windows.FPSWindow;
import zero1hd.rhythmbullet.ui.windows.PauseMenu;

public class GameHUD extends Stage {
	private Label score;
	private ImageButton pause;
	private boolean paused;
	private FPSWindow fpsWindow;
	private PauseMenu pauseMenu;
	private HealthBar healthBar;
	
	private Music music;
	
	public GameHUD(Skin skin, int maxHealth) {
		super();
		score = new Label("Score: 0", skin, "default-font", Color.WHITE);
		score.setPosition(10f, Gdx.graphics.getHeight()-score.getHeight() - 10f);
		addActor(score);
		
		pause = new ImageButton(skin.getDrawable("pause"),
				skin.getDrawable("pause-down"));
		pause.setPosition(Gdx.graphics.getWidth() - pause.getWidth() - 15f,
				Gdx.graphics.getHeight() - pause.getHeight() - 15f);
		pause.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				setPaused(true);
			}
		});
		addActor(pause);
		
		fpsWindow = new FPSWindow("FPS", skin);
		fpsWindow.setPosition(15f, 15f);
		
		pauseMenu = new PauseMenu(skin);
		pauseMenu.getResumeButton().addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				setPaused(false);
			}
		});
		pauseMenu.setPosition((Gdx.graphics.getWidth()-pauseMenu.getWidth())/2f, (Gdx.graphics.getHeight()-pauseMenu.getHeight())/2f);
		
		healthBar = new HealthBar(skin, maxHealth);
		healthBar.setSize(20f, Gdx.graphics.getHeight()/3);
		healthBar.setHealth(maxHealth);
		healthBar.setPosition(Gdx.graphics.getWidth()-healthBar.getWidth() -10f, (Gdx.graphics.getHeight()-healthBar.getHeight())/2f);
		addActor(healthBar);
	}
	
	/**
	 * sets the label for scoring to the designated score
	 * @param score designated score
	 */
	public void setScore(int score) {
		this.score.setText("Score: " + score);
	}
	
	/**
	 * returns current score by use of substring
	 * @return the current score value
	 */
	public int getScore() {
		return Integer.valueOf(score.getText().substring(7));
	}
	
	public void setPaused(boolean paused) {
		if (paused) {
			addActor(pauseMenu);
			music.pause();
		} else {
			pauseMenu.remove();
			music.play();
		}
		this.paused = paused;
	}
	
	/**
	 * toggle's between the two states of paused and unpaused
	 * @return whatever the new game state is (true for paused)
	 */
	public boolean togglePause() {
		if (isPaused()) {
			setPaused(false);
		} else {
			setPaused(true);
		}
		return isPaused();
	}
	
	public boolean isPaused() {
		return paused;
	}
	
	@Override
	public boolean keyUp(int keycode) {
		switch (keycode) {
		case Keys.F3:
			if (fpsWindow.hasParent()) {
				fpsWindow.remove();
			} else {
				addActor(fpsWindow);
			}
			break;
		case Keys.ESCAPE:
			togglePause();
			break;
		default:
			break;
		}
		return false;
	}
	
	public void setMusic(Music music) {
		this.music = music;
	}
}
