package zero1hd.rhythmbullet.ui.stages;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Preferences;
import com.badlogic.gdx.assets.AssetManager;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.Pixmap.Format;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.glutils.FrameBuffer;
import com.badlogic.gdx.graphics.glutils.ShaderProgram;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.utils.viewport.FitViewport;

import zero1hd.rhythmbullet.RhythmBullet;
import zero1hd.rhythmbullet.audio.map.EntitySpawnInfo;
import zero1hd.rhythmbullet.audio.map.GamePlayMap;
import zero1hd.rhythmbullet.audio.map.MapWindowData;
import zero1hd.rhythmbullet.controls.KeyMap;
import zero1hd.rhythmbullet.entity.CollisionDetector;
import zero1hd.rhythmbullet.entity.Entity;
import zero1hd.rhythmbullet.entity.EntityManager;
import zero1hd.rhythmbullet.entity.ally.Laser;
import zero1hd.rhythmbullet.entity.ally.PolyJetEntity;
import zero1hd.rhythmbullet.entity.coordinator.CoordinatorManager;


public class GamePlayArea extends Stage {
	public PolyJetEntity polyjet;
	private GamePlayMap audioMap;
	
	public CoordinatorManager cm;
	public EntityManager em;
	private CollisionDetector collisionDetector;
	
	private int maxHealth = 100;
	
	private float yTeleport = RhythmBullet.GAME_AREA_HEIGHT/2;
	
	private int score;
	
	private ShaderProgram glowShader;
	private FrameBuffer blurTarget;
	TextureRegion fboRegion;
	private int fboSize;
	
	public GamePlayArea(AssetManager assetManager, Preferences prefs) {
		super(new FitViewport(RhythmBullet.GAME_AREA_WIDTH, RhythmBullet.GAME_AREA_HEIGHT));
		Gdx.app.debug("Game Area", "new area created");
		
		polyjet = new PolyJetEntity(assetManager, 25f, 25f, "standard");
		em = new EntityManager(assetManager, prefs, this);
		cm = new CoordinatorManager(em);
		collisionDetector = new CollisionDetector(em.activeEnemies, em.activeAllies, assetManager, prefs);
		em.activeAllies.add(polyjet);
		addActor(polyjet);
	}
	
	public void setAudioMap(GamePlayMap audioMap) {
		this.audioMap = audioMap;
	}
	/**
	 * needs to be called right after set as screen (should be called in show method).
	 * @param prefs
	 */
	public void loadShaders(Preferences prefs) {
		if (prefs.getBoolean("glow shader")) {
			Gdx.app.debug("Shader", "using glow shader");
			glowShader = new ShaderProgram(Gdx.files.internal("shaders/glow.vsh"), Gdx.files.internal("shaders/glow.fsh"));
			if (!glowShader.isCompiled()) {
				System.err.println(glowShader.getLog());
				System.exit(0);
			}
			if (glowShader.getLog().length() != 0) {
				System.out.println(glowShader.getLog());
			}
			
			if (Gdx.graphics.getWidth() < 1024) {
				fboSize = 1024;
			} else if (Gdx.graphics.getWidth() < 2048) {
				fboSize = 2048;
			} else {
				fboSize = 4096;
			}
			
			blurTarget = new FrameBuffer(Format.RGBA8888, fboSize, fboSize, false);

			fboRegion = new TextureRegion(blurTarget.getColorBufferTexture());
			fboRegion.flip(false, true);
			
		}
		
		ShaderProgram.pedantic = false;
	}
	
	@Override
	public void draw() {
		//TODO batch draw background
		if (glowShader != null) {
			blurTarget.begin();

			Gdx.gl.glClearColor(0f, 0f, 0f, 0f);
			Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
			getBatch().setBlendFunction(-1, -1);
			Gdx.gl20.glBlendFuncSeparate(GL20.GL_ONE, GL20.GL_ONE_MINUS_SRC_ALPHA,GL20.GL_ONE, GL20.GL_DST_ALPHA);
			super.draw();
			collisionDetector.renderParticles(getBatch(), Gdx.graphics.getDeltaTime(), true);
			blurTarget.end(getViewport().getScreenX(), getViewport().getScreenY(), getViewport().getScreenWidth(), getViewport().getScreenHeight());
			getBatch().setBlendFunction(GL20.GL_SRC_ALPHA, GL20.GL_ONE_MINUS_SRC_ALPHA);
			
			
			getBatch().setBlendFunction(GL20.GL_ONE, GL20.GL_ONE_MINUS_SRC_ALPHA);
			getBatch().begin();
			getBatch().setShader(glowShader);
			getBatch().draw(fboRegion, 0f, 0f, RhythmBullet.GAME_AREA_WIDTH, RhythmBullet.GAME_AREA_HEIGHT);
			getBatch().setShader(null);
			getBatch().end();

		} else {
			super.draw();
		}
	}
	
	@Override
	public void act(float delta) {
		MapWindowData mwd;
		if (audioMap != null && audioMap.getMusicData().getPlaybackMusic().isPlaying()) {
			audioMap.getMusicData().readIndexUpdate();
			if ((mwd = audioMap.getCurrentWindowBasedOnIndex()) != null) {
				EntitySpawnInfo[] currentSpawnInfo = mwd.getArray();
				if (currentSpawnInfo != null) {
					for (int i = 0; i < currentSpawnInfo.length; i++) {
						Entity entity = currentSpawnInfo[i].getEntityToSpawn().buildEntity();
						if (currentSpawnInfo[i].getEntityCoordinator() != null) {
							entity.setCoordinator(currentSpawnInfo[i].getEntityCoordinator().buildCoordinator());
						}
						entity.init(currentSpawnInfo[i].parameters);
						addActor(entity);
					}
				}
			}
		}

		collisionDetector.collisionCheck();
		
		if (polyjet.getX() <= 1) {
			polyjet.moveLeft = false;
			polyjet.setX(1f);
		}
		
		if (polyjet.getX() >= RhythmBullet.GAME_AREA_WIDTH-1-polyjet.getWidth()) {
			polyjet.moveRight = false;
			polyjet.setX(RhythmBullet.GAME_AREA_WIDTH-1f-polyjet.getWidth());
		}
		
		if (polyjet.getY() >= RhythmBullet.GAME_AREA_HEIGHT - 1 - polyjet.getHeight()) {
			polyjet.moveUp = false;
			polyjet.setY(RhythmBullet.GAME_AREA_HEIGHT - 1 - polyjet.getHeight());
		}
		
		if (polyjet.getY() <= 1) {
			polyjet.moveDown = false;
			polyjet.setY(1f);
		}
		super.act(delta);
	}
	
	public void addScore (int score) {
		this.score += score;
		polyjet.health += score;
		if (polyjet.health > maxHealth) {
			polyjet.health = maxHealth;
		}
	}

	@Override
	public boolean keyDown(int keycode) {
		if (keycode == KeyMap.left) {
			polyjet.moveLeft = true;
		}
		if (keycode == KeyMap.right) {
			polyjet.moveRight = true;
		}
		
		if (keycode == KeyMap.up) {
			polyjet.moveUp = true;
		}
		
		if (keycode == KeyMap.down) {
			polyjet.moveDown = true;
		}
		
		if (keycode == KeyMap.accelerate) {
			polyjet.accelerate = true;
		}
		return false;
	}
	
	@Override
	public boolean keyUp(int keycode) {
		if (keycode == KeyMap.left) {
			polyjet.moveLeft = false;
		}
		if (keycode == KeyMap.right) {
			polyjet.moveRight = false;
		}
		if (keycode == KeyMap.up) {
			polyjet.moveUp = false;
		}
		if (keycode == KeyMap.down) {
			polyjet.moveDown = false;
		}
		
		if (keycode == KeyMap.accelerate) {
			polyjet.accelerate = false;
		}
		
		if (keycode == KeyMap.shoot) {
			Laser laser	= em.laser.buildEntity();
			laser.init(polyjet.getX() + polyjet.getWidth()/2f, polyjet.getY() + polyjet.getHeight()+1f, 60f);
			addActor(laser);
		}
		return false;
	}

	public PolyJetEntity getPolyjet() {
		return polyjet;
	}
	
	public int getScore() {
		return score;
	}
	
	public int getMaxHealth() {
		return maxHealth;
	}
	public float getyTeleport() {
		return yTeleport;
	}
	
	@Override
	public void dispose() {
		if (glowShader != null) {
			blurTarget.dispose();
			glowShader.dispose();
		}
		super.dispose();
	}
	
}
